import '../assets/css/sdpi.css';
import '../assets/css/styles.css';
import { default as EventEmitter } from 'eventemitter3';
import { isSomething } from 'ts-type-guards';
import Details from "./elements/Details";
import HtmlElement from "./elements/details/HtmlElement";
import HtmlElementInterface from "./HtmlElementInterface";
import Dropdown from './elements/Dropdown';
import Input from './elements/Input';
function isValueElementBag(value) {
    return (isSomething(value)
        && value.name !== undefined
        && value.element !== undefined);
}
export default class FormBuilder {
    constructor(initialData) {
        this.elementBag = [];
        this.eventEmitter = new EventEmitter();
        this.initialData = initialData;
    }
    /**
     * Returns the currently set formdata (or settings) as an object
     */
    getFormData() {
        const data = this.initialData;
        const valueElements = this.elementBag.filter(isValueElementBag);
        for (const element of valueElements) {
            data[element.name] = element.element.value;
        }
        return data;
    }
    /**
     * Allows to overwrite the formdata with whatever is passed
     */
    setFormData(data) {
        const valueElements = this.elementBag.filter(isValueElementBag);
        for (const element of valueElements) {
            element.element.setValue(data[element.name]);
        }
    }
    /**
     * Builds the form and appends all its elements to the passed html element
     */
    appendTo(element) {
        for (const formElement of this.elementBag) {
            if (isValueElementBag(formElement)) {
                element.append(formElement.element.getHtmlElement());
            }
            else {
                element.append(formElement.getHtmlElement());
            }
        }
    }
    // TODO: wtf is the grouping stuff for? does it even work?
    /**
     * Adds a new form element to the form
     * @param {string} name
     * @param value A element or a group of elements (created by the create* methods)
     */
    addElement(name, value) {
        this.elementBag.push({ element: value, name });
        value.setValue(this.initialData[name]);
    }
    /**
     * Allows to add an arbitrary object that returns a html element which then will be added to the form.
     * @param {HtmlElementInterface} element
     */
    addHtml(element) {
        this.elementBag.push(element);
    }
    /**
     * Adds a simple HTMLElement to the form
     */
    addHtmlElement(element) {
        this.elementBag.push(new HtmlElement(element));
    }
    /** Creates a text input element */
    createInput() {
        return this.addEventsToElement(new Input());
    }
    /** Creates a drowpdown (or select) element */
    createDropdown() {
        return this.addEventsToElement(new Dropdown());
    }
    /** Creates a collapsible detail element to output informational text */
    createDetails() {
        const details = new Details();
        details.onLinkClick((link) => this.eventEmitter.emit('click-link', link));
        return details;
    }
    /**
     * Allows to attach an eventlistener to formbuilder events
     * @param {'change-settings' | 'click-link'} event
     * @param callback The function that will get called once the event occurs
     */
    on(event, callback) {
        this.eventEmitter.on(event, callback);
    }
    addEventsToElement(element) {
        element.on('change-value', () => this.onChangeElementValue());
        this.eventEmitter.on('change-settings', () => element.reportSettingsChange());
        return element;
    }
    onChangeElementValue() {
        this.eventEmitter.emit('change-settings');
    }
}
